#include "config.h"
#include "adc.h"
#include "adcext.h"
#include "dac.h"
#include "uart.h"
#include "timer.h"
#include <stdio.h>
#include <math.h>

int send_data = 0;
uint16_t send_adc, send_dac;

/* Outside these thresholds, we step the DAC value */
#define ADC_MIN 0x074b
#define ADC_MAX 0x08b3

/* Max/min DAC output value */
#define DAC_MIN 0x0000
#define DAC_MAX 0xffff

/* A DAC step is this many counts */
#define DAC_STEP 0x0080

uint16_t dac_cmd = ((uint32_t)DAC_MAX + DAC_MIN) / 2;

/* 128 KHz, after each ADC sample */
void fast_callback(void)
{

}

/* 8 KHz, after DMA buffer filled */
void slow_callback(void)
{
	/* Get most recent sample. */
	uint16_t v = adc_dmabuf[adc_offset];

	/* Send most recent sample to PC */
	send_adc = v;
	send_dac = dac_cmd;
	send_data = 1;

	/* Update DAC if necessary */
	/* TODO: Increase step depending on how far from ADC_MIN/ADC_MAX we are */
	if (v < ADC_MIN && dac_cmd < (DAC_MAX - DAC_STEP)) {
		dac_cmd += DAC_STEP;
		dac_write(dac_cmd);
	} else if (v > ADC_MAX && dac_cmd > (DAC_MIN + DAC_STEP)) {
		dac_cmd -= DAC_STEP;
		dac_write(dac_cmd);
	}
}

void send_to_pc(void)
{
	/* Sent data format:
  	   1Aaa aaaa  0aaa aaDd  0ddd dddd 0ddd dddd
	   Aaaaaaaaaaaa = 12-bit ADC value
       Dddddddddddddddd = 16-bit DAC command
	*/
	uart1_put(0x80 | ((send_adc & 0x0FE0) >> 5));
	uart1_put(((send_adc & 0x001F) << 2) | ((send_dac & 0xC000) >> 14));
	uart1_put((send_dac & 0x3F80) >> 7);
	uart1_put((send_dac & 0x007F));
}

int main(void)
{
	config_init();
	uart1_init(500000);
	adcext_init();
	dac_init();
	adc_init();
//	adc_adc_callback = fast_callback;
	adc_dma_callback = slow_callback;

/*
	{ 
		int16_t i = 0;
		while(1) {
			i++;
			dac_write(i);
			uart1_put_hex16(i);
			uart1_crlf();
		}
	}
*/
			
	while(1) {
		if (send_data) {
			send_to_pc();
			send_data = 0;
		}
	}
}
